﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace CH372UpDown
{
    //由于vc版本的头文件中含有部分宏定义，在C#中无法实现（其中一部分转为常量），完整版请参考vc版CH341DLL.H
    [StructLayout(LayoutKind.Explicit, CharSet = CharSet.Ansi, Size = 8)]
    public struct mUSB_SETUP_PKT
    {				// USB控制传输的建立阶段的数据请求包结构
        [FieldOffset(0)]
        public byte mUspReqType;				// 00H 请求类型
        [FieldOffset(1)]
        public byte mUspRequest;				// 01H 请求代码

        [FieldOffset(2)]
        public byte mUspValueLow;				// 02H 值参数低字节
        [FieldOffset(3)]
        public byte mUspValueHigh;				// 03H 值参数高字节
        [FieldOffset(2)]
        public UInt16 mUspValue;					// 02H-03H 值参数

        [FieldOffset(4)]
        public byte mUspIndexLow;				// 04H 索引参数低字节
        [FieldOffset(5)]
        public byte mUspIndexHigh;				// 05H 索引参数高字节
        [FieldOffset(4)]
        public UInt16 mUspIndex;					// 04H-05H 索引参数
        [FieldOffset(6)]
        public UInt16 mLength;					// 06H-07H 数据阶段的数据长度
    }
    /*
     mWIN32_COMMAND结构体C++定义如下
     
     typedef	struct	_WIN32_COMMAND {				// 定义WIN32命令接口结构
         union	{
             ULONG		mFunction;					// 输入时指定功能代码或者管道号
             NTSTATUS	mStatus;					// 输出时返回操作状态
         };
         ULONG			mLength;					// 存取长度,返回后续数据的长度
         union	{
             mUSB_SETUP_PKT	mSetupPkt;				// USB控制传输的建立阶段的数据请求
             UCHAR			mBuffer[ mCH341_PACKET_LENGTH ];	// 数据缓冲区,长度为0至255B
         };
     } mWIN32_COMMAND, *mPWIN32_COMMAND;

    C#中union功能由FieldOffset()代替

    由于上述结构体中最后一个union可能引起错误“对象字段由一个非对象字段不正确地对齐或重叠”
    因此将上述结构体拆分成2个结构体mWIN32_COMMAND_USB_SETUP_PKT与mWIN32_COMMAND_mBuffer，选其一即可。
    CH341DriverCommand函数将重载

     */
    [StructLayout(LayoutKind.Explicit, CharSet = CharSet.Ansi)]
    public struct mWIN32_COMMAND_USB_SETUP_PKT
    {				// 定义WIN32命令接口结构
        [FieldOffset(0)]
        public UInt32 mFunction;					// 输入时指定功能代码或者管道号
        [FieldOffset(0)]
        public Int32 mStatus;					// 输出时返回操作状态
        [FieldOffset(4)]
        public UInt32 mLength;					// 存取长度,返回后续数据的长度
        [FieldOffset(8)]
        public mUSB_SETUP_PKT mSetupPkt;				// USB控制传输的建立阶段的数据请求


    }
    [StructLayout(LayoutKind.Explicit, CharSet = CharSet.Ansi)]
    public struct mWIN32_COMMAND_mBuffer
    {				// 定义WIN32命令接口结构
        [FieldOffset(0)]
        public UInt32 mFunction;					// 输入时指定功能代码或者管道号
        [FieldOffset(0)]
        public Int32 mStatus;					// 输出时返回操作状态
        [FieldOffset(4)]
        public UInt32 mLength;					// 存取长度,返回后续数据的长度

        [MarshalAs(UnmanagedType.ByValArray, SizeConst = CH375DLL.mCH375_PACKET_LENGTH), FieldOffset(8)]
        public byte[] mBuffer;	// 数据缓冲区,长度为0至255B

    }
    class CH375DLL
    {
        public const int        mCH375_PACKET_LENGTH =	64;			// CH375支持的数据包的长度
        public const int        mCH375_PKT_LEN_SHORT = 8;			// CH375支持的短数据包的长度
        // WIN32应用层接口命令
        //public const int		IOCTL_CH375_COMMAND		( FILE_DEVICE_UNKNOWN << 16 | FILE_ANY_ACCESS << 14 | 0x0f37 << 2 | METHOD_BUFFERED )	// 专用接口

        //public const int		mWIN32_COMMAND_HEAD		mOFFSET( mWIN32_COMMAND, mBuffer )	// WIN32命令接口的头长度

        public const int		mCH375_MAX_NUMBER	=	16;			// 最多同时连接的CH375数

        public const int		mMAX_BUFFER_LENGTH	=	0x1000;		// 数据缓冲区最大长度4096

       // public const int		mMAX_COMMAND_LENGTH		( mWIN32_COMMAND_HEAD + mMAX_BUFFER_LENGTH )	// 最大数据长度加上命令结构头的长度

        public const int		mDEFAULT_BUFFER_LEN	=	0x0400;		// 数据缓冲区默认长度1024

       // public const int		mDEFAULT_COMMAND_LEN	( mWIN32_COMMAND_HEAD + mDEFAULT_BUFFER_LEN )	// 默认数据长度加上命令结构头的长度


        // CH375端点地址
        public const int		mCH375_ENDP_INTER_UP=	0x81;		// CH375的中断数据上传端点的地址
        public const int		mCH375_ENDP_AUX_DOWN=	0x01;		// CH375的辅助数据下传端点的地址
        public const int		mCH375_ENDP_DATA_UP	=	0x82;		// CH375的数据块上传端点的地址
        public const int		mCH375_ENDP_DATA_DOWN=	0x02;		// CH375的数据块下传端点的地址


        // 设备层接口提供的管道操作命令
        public const int		mPipeDeviceCtrl	=		0x00000004;	// CH375的综合控制管道
        public const int		mPipeInterUp=			0x00000005;	// CH375的中断数据上传管道
        public const int		mPipeDataUp	=			0x00000006;	// CH375的数据块上传管道
        public const int		mPipeDataDown   =		0x00000007;	// CH375的数据块下传管道
        public const int		mPipeAuxDown=			0x00000008;	// CH375的辅助数据下传管道

        // 应用层接口的功能代码
        public const int		mFuncNoOperation=		0x00000000;	// 无操作
        public const int		mFuncGetVersion	=		0x00000001;	// 获取驱动程序版本号
        public const int		mFuncGetConfig	=		0x00000002;	// 获取USB设备配置描述符
        public const int		mFuncSetExclusive=		0x0000000b;	// 设置独占使用
        public const int		mFuncResetDevice	=	0x0000000c;	// 复位USB设备
        public const int		mFuncResetPipe		=	0x0000000d;	// 复位USB管道
        public const int		mFuncAbortPipe		=	0x0000000e;	// 取消USB管道的数据请求
        public const int		mFuncSetTimeout		=	0x0000000f;	// 设置USB通讯超时
        public const int		mFuncBufferMode		=	0x00000010;	// 设定缓冲上传模式及查询缓冲区中的数据长度
        public const int		mFuncBufferModeDn	=	0x00000011;	// 设定缓冲下传模式及查询缓冲区中的数据长度


        // USB设备标准请求代码
        public const int		mUSB_CLR_FEATURE=		0x01;
        public const int		mUSB_SET_FEATURE=		0x03;
        public const int		mUSB_GET_STATUS	=		0x00;
        public const int		mUSB_SET_ADDRESS=		0x05;
        public const int		mUSB_GET_DESCR	=		0x06;
        public const int		mUSB_SET_DESCR	=		0x07;
        public const int		mUSB_GET_CONFIG	=		0x08;
        public const int		mUSB_SET_CONFIG	=		0x09;
        public const int		mUSB_GET_INTERF	=		0x0a;
        public const int		mUSB_SET_INTERF	=		0x0b;
        public const int		mUSB_SYNC_FRAME	=		0x0c;

        // CH375控制传输的厂商专用请求类型
        public const int		mCH375_VENDOR_READ	=	0xc0;		// 通过控制传输实现的CH375厂商专用读操作
        public const int		mCH375_VENDOR_WRITE	=	0x40;		// 通过控制传输实现的CH375厂商专用写操作

        // CH375控制传输的厂商专用请求代码
        public const int		mCH375_SET_CONTROL	=	0x51;		// 输出控制信号
        public const int		mCH375_GET_STATUS	=	0x52;		// 输入状态信号

        // 寄存器的位定义
        public const int		mBitInputRxd	=		0x02;		// 只读,RXD#引脚输入状态,1:高电平,0:低电平
        public const int		mBitInputReq	=		0x04;		// 只读,REQ#引脚输入状态,1:高电平,0:低电平


        // 直接输入的状态信号的位定义
        public const int		mStateRXD		=		0x00000200;	// RXD#引脚输入状态,1:高电平,0:低电平
        public const int		mStateREQ		=		0x00000400;	// REQ#引脚输入状态,1:高电平,0:低电平
        public const int		MAX_DEVICE_PATH_SIZE	=128	;		// 设备名称的最大字符数
        public const int		MAX_DEVICE_ID_SIZE		=64	;		// 设备ID的最大字符数

        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        public delegate void	mPCH375_INT_ROUTINE(  // 中断服务回调程序
	                            byte[]			iBuffer );  // 指向一个缓冲区,提供当前的中断特征数据

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375OpenDevice")]
        public static extern IntPtr 	CH375OpenDevice(  // 打开CH375设备,返回句柄,出错则无效
	        uint			iIndex );  // 指定CH375设备序号,0对应第一个设备,-1则自动搜索一个可以被打开的设备并返回序号

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375CloseDevice")]
        public static extern void	    CH375CloseDevice(  // 关闭CH375设备
	        uint			iIndex );  // 指定CH375设备序号

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375GetVersion")]
        public static extern uint		CH375GetVersion( );  // 获得DLL版本号,返回版本号

        //重载函数
        [DllImport("CH375DLL.DLL", EntryPoint = "CH375DriverCommand")]
        public static extern uint		CH375DriverCommand(  // 直接传递命令给驱动程序,出错则返回0,否则返回数据长度
	        uint			iIndex,  // 指定CH375设备序号,V1.6以上DLL也可以是设备打开后的句柄
	        ref mWIN32_COMMAND_mBuffer	ioCommand );  // 命令结构的指针

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375DriverCommand")]
        public static extern uint		CH375DriverCommand(  // 直接传递命令给驱动程序,出错则返回0,否则返回数据长度
	        uint			iIndex,  // 指定CH375设备序号,V1.6以上DLL也可以是设备打开后的句柄
	        ref mWIN32_COMMAND_USB_SETUP_PKT	ioCommand );  // 命令结构的指针
        // 该程序在调用后返回数据长度,并且仍然返回命令结构,如果是读操作,则数据返回在命令结构中,
        // 返回的数据长度在操作失败时为0,操作成功时为整个命令结构的长度,例如读一个字节,则返回mWIN32_COMMAND_HEAD+1,
        // 命令结构在调用前,分别提供:管道号或者命令功能代码,存取数据的长度(可选),数据(可选)
        // 命令结构在调用后,分别返回:操作状态代码,后续数据的长度(可选),
        //   操作状态代码是由WINDOWS定义的代码,可以参考NTSTATUS.H,
        //   后续数据的长度是指读操作返回的数据长度,数据存放在随后的缓冲区中,对于写操作一般为0

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375GetDrvVersion")]
        public static extern uint		CH375GetDrvVersion( );  // 获得驱动程序版本号,返回版本号,出错则返回0

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375ResetDevice")]
        public static extern bool		CH375ResetDevice(  // 复位USB设备
	        uint			iIndex );  // 指定CH375设备序号

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375GetDeviceDescr")]
        public static extern bool		CH375GetDeviceDescr(  // 读取设备描述符
	        uint			iIndex,  // 指定CH375设备序号
	        byte[]			oBuffer,  // 指向一个足够大的缓冲区,用于保存描述符
	        ref UInt32			ioLength );  // 指向长度单元,输入时为准备读取的长度,返回后为实际读取的长度

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375GetConfigDescr")]
        public static extern bool		CH375GetConfigDescr(  // 读取配置描述符
	        uint			iIndex,  // 指定CH375设备序号
	        byte[]			oBuffer,  // 指向一个足够大的缓冲区,用于保存描述符
	        ref uint			ioLength );  // 指向长度单元,输入时为准备读取的长度,返回后为实际读取的长度

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375SetIntRoutine")]
        public static extern bool		CH375SetIntRoutine(  // 设定中断服务程序
	        uint			iIndex,  // 指定CH375设备序号
	        mPCH375_INT_ROUTINE	iIntRoutine );  // 指定中断服务回调程序,为NULL则取消中断服务,否则在中断时调用该程序

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375ReadInter")]
        public static extern bool		CH375ReadInter(  // 读取中断数据
	        uint			iIndex,  // 指定CH375设备序号
	        byte[]			oBuffer,  // 指向一个足够大的缓冲区,用于保存读取的中断数据
	        ref uint			ioLength );  // 指向长度单元,输入时为准备读取的长度,返回后为实际读取的长度

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375AbortInter")]
        public static extern bool		CH375AbortInter(  // 放弃中断数据读操作
	        uint			iIndex );  // 指定CH375设备序号

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375ReadData")]
        public static extern bool		CH375ReadData(  // 读取数据块
	        uint			iIndex,  // 指定CH375设备序号
	        byte[]			oBuffer,  // 指向一个足够大的缓冲区,用于保存读取的数据
	        ref uint			ioLength );  // 指向长度单元,输入时为准备读取的长度,返回后为实际读取的长度

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375AbortRead")]
        public static extern bool		CH375AbortRead(  // 放弃数据块读操作
	        uint			iIndex );  // 指定CH375设备序号

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375WriteData")]
        public static extern bool		CH375WriteData(  // 写出数据块
	        uint			iIndex,  // 指定CH375设备序号
	        byte[]			iBuffer,  // 指向一个缓冲区,放置准备写出的数据
	        ref uint			ioLength );  // 指向长度单元,输入时为准备写出的长度,返回后为实际写出的长度

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375AbortWrite")]
        public static extern bool		CH375AbortWrite(  // 放弃数据块写操作
	        uint			iIndex );  // 指定CH375设备序号

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375WriteRead")]
        public static extern bool		CH375WriteRead(  // 先写出标准的数据块(命令),再读取标准的数据块(应答)
	        uint			iIndex,  // 指定CH375设备序号
	        byte[]			iBuffer,  // 指向一个缓冲区,放置准备写出的数据,长度不大于mCH375_PACKET_LENGTH
	        byte[]			oBuffer,  // 指向一个足够大的缓冲区,长度不小于mCH375_PACKET_LENGTH,用于保存读取的数据
	        ref uint			ioLength );  // 指向长度单元,不大于mCH375_PACKET_LENGTH,输入时为准备写出的长度,返回后为实际读取的长度

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375GetStatus")]
        public static extern bool		CH375GetStatus(  // 通过CH375直接输入数据和状态
	        uint			iIndex,  // 指定CH375设备序号
	        ref uint			iStatus );  // 指向一个双字单元,用于保存状态数据
        // 位7-位0对应CH375的D7-D0引脚,位9对应CH375的RXD#引脚,位10对应CH375的REQ#引脚

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375SetTimeout")]
        public static extern bool		CH375SetTimeout(  // 设置USB数据读写的超时
	        uint			iIndex,  // 指定CH375设备序号
	        uint			iWriteTimeout,  // 指定USB写出数据块的超时时间,以毫秒mS为单位,0xFFFFFFFF指定不超时(默认值)
	        uint			iReadTimeout );  // 指定USB读取数据块的超时时间,以毫秒mS为单位,0xFFFFFFFF指定不超时(默认值)

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375WriteAuxData")]
        public static extern bool		CH375WriteAuxData(  // 写出辅助数据
	        uint			iIndex,  // 指定CH375设备序号
	        byte[]			iBuffer,  // 指向一个缓冲区,放置准备写出的数据
	        ref uint			ioLength );  // 指向长度单元,输入时为准备写出的长度,返回后为实际写出的长度

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375SetExclusive")]
        public static extern bool		CH375SetExclusive(  // 设置独占使用当前CH375设备
	        uint			iIndex,  // 指定CH375设备序号
	        uint			iExclusive );  // 为0则设备可以共享使用,非0则独占使用

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375GetUsbID")]
        public static extern uint		CH375GetUsbID(  // 获取USB设备ID,返回数据中,低16位为厂商ID,高16位为产品ID,错误时返回全0(无效ID)
	        uint			iIndex );  // 指定CH375设备序号

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375GetDeviceName")]
        public static extern string		CH375GetDeviceName(  // 返回指向CH375设备名称的缓冲区,出错则返回NULL
	        uint			iIndex );  // 指定CH375设备序号,0对应第一个设备

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375SetBufUpload")]
        public static extern bool	CH375SetBufUpload(  // 设定内部缓冲上传模式
	        uint			iIndex,  // 指定CH375设备序号,0对应第一个设备
	        uint			iEnableOrClear );  // 为0则禁止内部缓冲上传模式,使用直接上传,非0则启用内部缓冲上传模式并清除缓冲区中的已有数据
        // 如果启用内部缓冲上传模式,那么CH375驱动程序创建线程自动接收USB上传数据到内部缓冲区,同时清除缓冲区中的已有数据,当应用程序调用CH375ReadData后将立即返回缓冲区中的已有数据

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375QueryBufUpload")]
         public static extern Int32		CH375QueryBufUpload(  // 查询内部上传缓冲区中的已有数据包个数,成功返回数据包个数,出错返回-1
	        uint			iIndex );  // 指定CH375设备序号,0对应第一个设备

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375SetBufDownload")]
        public static extern bool		CH375SetBufDownload(  // 设定内部缓冲下传模式
	        uint			iIndex,  // 指定CH375设备序号,0对应第一个设备
	        uint			iEnableOrClear );  // 为0则禁止内部缓冲下传模式,使用直接下传,非0则启用内部缓冲下传模式并清除缓冲区中的已有数据
        // 如果启用内部缓冲下传模式,那么当应用程序调用CH375WriteData后将仅仅是将USB下传数据放到内部缓冲区并立即返回,而由CH375驱动程序创建的线程自动发送直到完毕

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375QueryBufDownload")]
        public static extern Int32		CH375QueryBufDownload(  // 查询内部下传缓冲区中的剩余数据包个数(尚未发送),成功返回数据包个数,出错返回-1
	        uint			iIndex );  // 指定CH375设备序号,0对应第一个设备

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375ResetInter")]
        public static extern bool		CH375ResetInter(  // 复位中断数据读操作
	        uint			iIndex );  // 指定CH375设备序号

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375ResetAux")]
        public static extern bool		CH375ResetAux(  // 复位辅助数据写操作
	        uint			iIndex );  // 指定CH375设备序号

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375ResetRead")]
        public static extern bool		CH375ResetRead(  // 复位数据块读操作
	        uint			iIndex );  // 指定CH375设备序号

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375ResetWrite")]
        public static extern bool		CH375ResetWrite(  // 复位数据块写操作
	        uint			iIndex );  // 指定CH375设备序号

        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        public  delegate void	 mPCH375_NOTIFY_ROUTINE  (  // 设备事件通知回调程序
	        uint			iEventStatus );  // 设备事件和当前状态(在下行定义): 0=设备拔出事件, 3=设备插入事件

        public const int		CH375_DEVICE_ARRIVAL	=	3;		// 设备插入事件,已经插入
        public const int		CH375_DEVICE_REMOVE_PEND=	1;		// 设备将要拔出
        public const int		CH375_DEVICE_REMOVE		=	0;		// 设备拔出事件,已经拔出

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375SetDeviceNotify")]
        public static extern bool		CH375SetDeviceNotify(  // 设定设备事件通知程序
	        uint					iIndex,  // 指定CH375设备序号,0对应第一个设备
	        string 					iDeviceID,  // 可选参数,指向字符串,指定被监控的设备的ID,字符串以\0终止
	        mPCH375_NOTIFY_ROUTINE	iNotifyRoutine );  // 指定设备事件回调程序,为NULL则取消事件通知,否则在检测到事件时调用该程序

        [DllImport("CH375DLL.DLL", EntryPoint = "CH375SetTimeoutEx")]
        public static extern bool		CH375SetTimeoutEx(  // 设置USB数据读写的超时
            uint iIndex,  // 指定CH375设备序号
            uint iWriteTimeout,  // 指定USB写出数据块的超时时间,以毫秒mS为单位,0xFFFFFFFF指定不超时(默认值)
            uint iReadTimeout,  // 指定USB读取数据块的超时时间,以毫秒mS为单位,0xFFFFFFFF指定不超时(默认值)
            uint iAuxTimeout,  // 指定USB辅助下传数据的超时时间,以毫秒mS为单位,0xFFFFFFFF指定不超时(默认值)
            uint iInterTimeout);  // 指定USB中断上传数据的超时时间,以毫秒mS为单位,0xFFFFFFFF指定不超时(默认值)

             }

}
